/**
 * @license @nomadreservations/ngx-codemirror
 * UNLICENSED
 * Copyright © 2017 Nomad Reservatoins, LLC. All Rights Reserved
 */

import { Component, EventEmitter, Injectable, Input, NgModule, NgZone, Output, ViewChild, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { ReplaySubject as ReplaySubject$1 } from 'rxjs/ReplaySubject';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Main Codemirror import, utilizing window's existence to determine if we're server side or not.
 */
const CodeMirror$1 = typeof window !== 'undefined' && typeof window.navigator !== 'undefined'
    ? require('codemirror')
    : undefined;
/**
 * Initialize Event for CodeMirror.Editor instance
 *
 * Holds a referencable pointer to the code mirror instance for users.
 */
class CodemirrorService {
    constructor() {
        /**
         * Codemirror instance subject
         *
         * Emits a refrence to the initialized CodeMirror.Editor once it's insantiated.
         */
        this.instance$ = new ReplaySubject$1();
    }
}
CodemirrorService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
CodemirrorService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Imports
/**
 * Main Codemirror import, utilizing window's existence to determine if we're server side or not.
 */
const CodeMirror = typeof window !== 'undefined' && typeof window.navigator !== 'undefined'
    ? require('codemirror')
    : undefined;
/**
 * CodeMirror component
 *
 * **Usage** :
 * ```html
 *   <ngx-codemirror [(ngModel)]="data" [config]="{...}" (init)="onInit" (blur)="onBlur" (focus)="onFocus" ...></ngx-codemirror>
 * ```
 */
class CodemirrorComponent {
    /**
     * Constructor
     *
     * @param {?} _codeMirror
     * @param {?} _zone NgZone injected for Initialization
     */
    constructor(_codeMirror, _zone) {
        this._codeMirror = _codeMirror;
        this._zone = _zone;
        /**
         * change output event, pass through from codemirror
         */
        this.change = new EventEmitter();
        /**
         * focus output event, pass through from codemirror
         */
        this.focus = new EventEmitter();
        /**
         * blur output event, pass through from codemirror
         */
        this.blur = new EventEmitter();
        /**
         * cursorActivity output event, pass through from codemirror
         */
        this.cursorActivity = new EventEmitter();
        /**
         * Value storage
         */
        this._value = '';
    }
    /**
     * Implements ControlValueAccessor.value
     * @return {?}
     */
    get value() { return this._value; }
    /**
     * Implements ControlValueAccessor.value
     * @param {?} v
     * @return {?}
     */
    set value(v) {
        if (v !== this._value) {
            this._value = v;
            this.onChange(v);
        }
    }
    /**
     * On component destroy
     * @return {?}
     */
    ngOnDestroy() {
    }
    /**
     * On component view init
     * @return {?}
     */
    ngAfterViewInit() {
        this.config = this.config || {};
        this.codemirrorInit(this.config);
    }
    /**
     * Value update process
     * @param {?} value
     * @return {?}
     */
    updateValue(value) {
        this.value = value;
        this.onTouched();
        this.change.emit(value);
    }
    /**
     * Implements ControlValueAccessor
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this._value = value || '';
        if (this._instance) {
            this._instance.setValue(this._value);
        }
    }
    /**
     * Change event trigger
     * @param {?} _
     * @return {?}
     */
    onChange(_) { }
    /**
     * Dirty/touched event trigger
     * @return {?}
     */
    onTouched() { }
    /**
     * Implements ControlValueAccessor.registerOnChange
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) { this.onChange = fn; }
    /**
     * Implements ControlValueAccessor.registerOnTouched
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) { this.onTouched = fn; }
    /**
     * Initialize codemirror
     * @param {?} config
     * @return {?}
     */
    codemirrorInit(config) {
        if (CodeMirror) {
            this._zone.runOutsideAngular(() => {
                this._instance = CodeMirror.fromTextArea(this.host.nativeElement, config);
                this._instance.setValue(this._value);
            });
            this._instance.on('change', () => {
                this.updateValue(this._instance.getValue());
            });
            this._instance.on('focus', (instance, event) => {
                this.focus.emit({ instance, event });
            });
            this._instance.on('cursorActivity', (instance) => {
                this.cursorActivity.emit({ instance });
            });
            this._instance.on('blur', (instance, event) => {
                this.blur.emit({ instance, event });
            });
            this._codeMirror.instance$.next(this._instance);
        }
    }
}
CodemirrorComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-codemirror',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        // tslint:disable-next-line:no-forward-ref
                        useExisting: forwardRef(() => CodemirrorComponent),
                        multi: true
                    }
                ],
                template: '<textarea #host></textarea>',
            },] },
];
/** @nocollapse */
CodemirrorComponent.ctorParameters = () => [
    { type: CodemirrorService, },
    { type: NgZone, },
];
CodemirrorComponent.propDecorators = {
    "config": [{ type: Input },],
    "change": [{ type: Output },],
    "focus": [{ type: Output },],
    "blur": [{ type: Output },],
    "cursorActivity": [{ type: Output },],
    "host": [{ type: ViewChild, args: ['host',] },],
    "value": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * \@angular 5.x+ SSR ready CodeMirror wrapping module.
 */
class CodemirrorModule {
}
CodemirrorModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    CodemirrorService
                ],
                declarations: [CodemirrorComponent],
                exports: [CodemirrorComponent],
                entryComponents: [CodemirrorComponent]
            },] },
];
/** @nocollapse */
CodemirrorModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { CodemirrorModule, CodemirrorComponent, CodemirrorService };
//# sourceMappingURL=ngx-codemirror.js.map
